module Hermes
  module RexClient
    class UpdateContactTask

      def update_contact(rex_entity, client = Hermes::RexClient::Client.new, yopmail = false, skip_update_groups = false)
        fields = rex_entity['fields']
        return unless fields['email'] && fields['id_cristal']  && ['cre'].include?(fields['rolecrelocal'].downcase)
        puts "updating #{fields['email']} #{fields}"

        if Rails.env.test? || yopmail
          fields['email'] = fields['email'].split('@')[0] + '@yopmail.com'
        end

        contact = User.find_by(id_cristal: fields['id_cristal']) unless fields['id_cristal'].blank?
        contact = User.find_by(email: fields['email'].downcase) unless contact
        
        if !contact
          puts 'create new user'
          contact = User.create(user_fields(fields).merge(password: Devise.friendly_token[0,20]))
        else
          puts 'update existing user'
          contact.update(user_fields(fields, contact.login))
        end

        contact.create_user_on_identity_provider
        
        unless skip_update_groups
          contact.group_ids = [] unless contact.group_ids
          [
            {fieldName: 'id_etablissement_cristal', classname: Hermes::Entity },
            # {fieldName: 'id_bailleur_cristal', classname: Hermes::Lessor },
          ].each do |type|
            query = fields[type[:fieldName]] if fields[type[:fieldName]].is_a? String
            query = {'$in': fields[type[:fieldName]]} if fields[type[:fieldName]].is_a? Array

            ids = type[:classname].where(id_cristal: query).pluck(:manager_group_id) 
            puts "updating on #{ids.length} ids"
            contact.group_ids = (contact.group_ids + (ids || [])).uniq
          end
        end
        
        contact.save!
        contact
      end


      def user_fields(fields, contact_login = nil)
        mobileNumber = fields['tel_pro'].blank? ? '1234567890' : fields['tel_pro']
        role = fields['rolecrelocal'].blank? ? 'cre' : fields['rolecrelocal'].downcase
        login = contact_login || fields['email'].downcase
        {
          username: login,
          first_name: fields['prenom'],
          last_name: fields['nom'],
          id_cristal: fields['id_cristal'],
          email: fields['email'].downcase,
          login: login,
          auth_roles: [role],
          phone_number: mobileNumber,
          provider: HERMES_CONFIG['identity_provider_key'],
          create_user_data: {
            gender: 'M',
            firstName: fields['prenom'],
            lastName: fields['nom'],
            email: fields['email'].downcase,
            login: login,
            mobileNumber: mobileNumber,
            roles: [role]
          },
        }
      end
    end
  end
end
